# autogenerated file, do not modify

from os import PathLike, curdir
from typing import Union

from flopy.mf6.data.mfdatautil import ArrayTemplateGenerator, ListTemplateGenerator
from flopy.mf6.mfpackage import MFChildPackages, MFPackage


class ModflowGwfsfr(MFPackage):
    """
    ModflowGwfsfr defines a SFR package.

    Parameters
    ----------
    storage : keyword
        keyword that activates storage contributions to the stream-flow routing package
        continuity equation.
    auxiliary : [string]
        defines an array of one or more auxiliary variable names.  there is no limit on
        the number of auxiliary variables that can be provided on this line; however,
        lists of information provided in subsequent blocks must have a column of data
        for each auxiliary variable name defined here.   the number of auxiliary
        variables detected on this line determines the value for naux.  comments cannot
        be provided anywhere on this line as they will be interpreted as auxiliary
        variable names.  auxiliary variables may not be used by the package, but they
        will be available for use by other parts of the program.  the program will
        terminate with an error if auxiliary variables are specified on more than one
        line in the options block.
    boundnames : keyword
        keyword to indicate that boundary names may be provided with the list of stream
        reach cells.
    print_input : keyword
        keyword to indicate that the list of stream reach information will be written
        to the listing file immediately after it is read.
    print_stage : keyword
        keyword to indicate that the list of stream reach {#2} will be printed to the
        listing file for every stress period in which 'head print' is specified in
        output control.  if there is no output control option and print_{#3} is
        specified, then {#2} are printed for the last time step of each stress period.
    print_flows : keyword
        keyword to indicate that the list of stream reach flow rates will be printed to
        the listing file for every stress period time step in which 'budget print' is
        specified in output control.  if there is no output control option and
        'print_flows' is specified, then flow rates are printed for the last time step
        of each stress period.
    save_flows : keyword
        keyword to indicate that stream reach flow terms will be written to the file
        specified with 'budget fileout' in output control.
    stage_filerecord : (stagefile)
        * stagefile : string
                name of the binary output file to write stage information.

    budget_filerecord : (budgetfile)
        * budgetfile : string
                name of the binary output file to write budget information.

    budgetcsv_filerecord : (budgetcsvfile)
        * budgetcsvfile : string
                name of the comma-separated value (CSV) output file to write budget summary
                information.  A budget summary record will be written to this file for each
                time step of the simulation.

    package_convergence_filerecord : (package_convergence_filename)
        * package_convergence_filename : string
                name of the comma spaced values output file to write package convergence
                information.

    timeseries : record ts6 filein ts6_filename
        Contains data for the ts package. Data can be passed as a dictionary to the ts
        package with variable names as keys and package data as values. Data for the
        timeseries variable is also acceptable. See ts package documentation for more
        information.
    observations : record obs6 filein obs6_filename
        Contains data for the obs package. Data can be passed as a dictionary to the
        obs package with variable names as keys and package data as values. Data for
        the observations variable is also acceptable. See obs package documentation for
        more information.
    mover : keyword
        keyword to indicate that this instance of the sfr package can be used with the
        water mover (mvr) package.  when the mover option is specified, additional
        memory is allocated within the package to store the available, provided, and
        received water.
    maximum_picard_iterations : integer
        integer value that defines the maximum number of streamflow routing picard
        iterations allowed when solving for reach stages and flows as part of the gwf
        formulate step. picard iterations are used to minimize differences in sfr
        package results between subsequent gwf picard (non-linear) iterations as a
        result of non-optimal reach numbering. if reaches are numbered in order, from
        upstream to downstream, maximum_picard_iterations can be set to 1 to reduce
        model run time. by default, maximum_picard_iterations is equal to 100.
    maximum_iterations : integer
        integer value that defines the maximum number of streamflow routing newton-
        raphson iterations allowed for a reach. by default, maximum_iterations is equal
        to 100. maximum_iterations would only need to be increased from the default
        value if one or more reach in a simulation has a large water budget error.
    maximum_depth_change : double precision
        real value that defines the depth closure tolerance. by default,
        maximum_depth_change is equal to :math:`1 times 10^{-5}`. the
        maximum_stage_change would only need to be increased or decreased from the
        default value if the water budget error for one or more reach is too small or
        too large, respectively.
    unit_conversion : double precision
        real value that is used to convert user-specified manning's roughness
        coefficients from seconds per meters:math:`^{1/3}` to model length and time
        units. a constant of 1.486 is used for flow units of cubic feet per second, and
        a constant of 1.0 is used for units of cubic meters per second. the constant
        must be multiplied by 86,400 when using time units of days in the simulation.
    length_conversion : double precision
        real value that is used to convert user-specified manning's roughness
        coefficients from meters to model length units. length_conversion should be set
        to 3.28081, 1.0, and 100.0 when using length units (length_units) of feet,
        meters, or centimeters in the simulation, respectively. length_conversion does
        not need to be specified if length_units are meters.
    time_conversion : double precision
        real value that is used to convert user-specified manning's roughness
        coefficients from seconds to model time units. time_conversion should be set to
        1.0, 60.0, 3,600.0, 86,400.0, and 31,557,600.0 when using time units
        (time_units) of seconds, minutes, hours, days, or years in the simulation,
        respectively. time_conversion does not need to be specified if time_units are
        seconds.
    dev_storage_weight : double precision
        real number value that defines the time weighting factor used to calculate the
        change in channel storage. storage_weight must have a value between 0.5 and 1.
        default storage_weight value is 1.
    nreaches : integer
        integer value specifying the number of stream reaches.  there must be nreaches
        entries in the packagedata block.
    packagedata : [list]
    crosssections : list
    connectiondata : [list]
    diversions : [list]
    initialstages : [list]
    perioddata : list

    """

    auxiliary = ArrayTemplateGenerator(("gwf6", "sfr", "options", "auxiliary"))
    stage_filerecord = ListTemplateGenerator(
        ("gwf6", "sfr", "options", "stage_filerecord")
    )
    budget_filerecord = ListTemplateGenerator(
        ("gwf6", "sfr", "options", "budget_filerecord")
    )
    budgetcsv_filerecord = ListTemplateGenerator(
        ("gwf6", "sfr", "options", "budgetcsv_filerecord")
    )
    package_convergence_filerecord = ListTemplateGenerator(
        ("gwf6", "sfr", "options", "package_convergence_filerecord")
    )
    ts_filerecord = ListTemplateGenerator(("gwf6", "sfr", "options", "ts_filerecord"))
    obs_filerecord = ListTemplateGenerator(("gwf6", "sfr", "options", "obs_filerecord"))
    packagedata = ListTemplateGenerator(("gwf6", "sfr", "packagedata", "packagedata"))
    crosssections = ListTemplateGenerator(
        ("gwf6", "sfr", "crosssections", "crosssections")
    )
    connectiondata = ListTemplateGenerator(
        ("gwf6", "sfr", "connectiondata", "connectiondata")
    )
    diversions = ListTemplateGenerator(("gwf6", "sfr", "diversions", "diversions"))
    initialstages = ListTemplateGenerator(
        ("gwf6", "sfr", "initialstages", "initialstages")
    )
    perioddata = ListTemplateGenerator(("gwf6", "sfr", "period", "perioddata"))
    package_abbr = "gwfsfr"
    _package_type = "sfr"
    dfn_file_name = "gwf-sfr.dfn"
    dfn = [
        ["header", "multi-package", "package-type advanced-stress-package"],
        [
            "block options",
            "name storage",
            "type keyword",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name auxiliary",
            "type string",
            "shape (naux)",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name boundnames",
            "type keyword",
            "shape",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name print_input",
            "type keyword",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name print_stage",
            "type keyword",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name print_flows",
            "type keyword",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name save_flows",
            "type keyword",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name stage_filerecord",
            "type record stage fileout stagefile",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
        ],
        [
            "block options",
            "name stage",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name stagefile",
            "type string",
            "preserve_case true",
            "shape",
            "in_record true",
            "reader urword",
            "tagged false",
            "optional false",
        ],
        [
            "block options",
            "name budget_filerecord",
            "type record budget fileout budgetfile",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
        ],
        [
            "block options",
            "name budget",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name fileout",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name budgetfile",
            "type string",
            "preserve_case true",
            "shape",
            "in_record true",
            "reader urword",
            "tagged false",
            "optional false",
        ],
        [
            "block options",
            "name budgetcsv_filerecord",
            "type record budgetcsv fileout budgetcsvfile",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
        ],
        [
            "block options",
            "name budgetcsv",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name budgetcsvfile",
            "type string",
            "preserve_case true",
            "shape",
            "in_record true",
            "reader urword",
            "tagged false",
            "optional false",
        ],
        [
            "block options",
            "name package_convergence_filerecord",
            "type record package_convergence fileout package_convergence_filename",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
        ],
        [
            "block options",
            "name package_convergence",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name package_convergence_filename",
            "type string",
            "shape",
            "in_record true",
            "reader urword",
            "tagged false",
            "optional false",
        ],
        [
            "block options",
            "name ts_filerecord",
            "type record ts6 filein ts6_filename",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
            "construct_package ts",
            "construct_data timeseries",
            "parameter_name timeseries",
        ],
        [
            "block options",
            "name ts6",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name filein",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name ts6_filename",
            "type string",
            "preserve_case true",
            "in_record true",
            "reader urword",
            "optional false",
            "tagged false",
        ],
        [
            "block options",
            "name obs_filerecord",
            "type record obs6 filein obs6_filename",
            "shape",
            "reader urword",
            "tagged true",
            "optional true",
            "construct_package obs",
            "construct_data observations",
            "parameter_name continuous",
        ],
        [
            "block options",
            "name obs6",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block options",
            "name obs6_filename",
            "type string",
            "preserve_case true",
            "in_record true",
            "tagged false",
            "reader urword",
            "optional false",
        ],
        [
            "block options",
            "name mover",
            "type keyword",
            "tagged true",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name maximum_picard_iterations",
            "type integer",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name maximum_iterations",
            "type integer",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name maximum_depth_change",
            "type double precision",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name unit_conversion",
            "type double precision",
            "reader urword",
            "optional true",
            "deprecated 6.4.2",
        ],
        [
            "block options",
            "name length_conversion",
            "type double precision",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name time_conversion",
            "type double precision",
            "reader urword",
            "optional true",
        ],
        [
            "block options",
            "name dev_storage_weight",
            "type double precision",
            "reader urword",
            "optional true",
        ],
        [
            "block dimensions",
            "name nreaches",
            "type integer",
            "reader urword",
            "optional false",
        ],
        [
            "block packagedata",
            "name packagedata",
            "type recarray ifno cellid rlen rwid rgrd rtp rbth rhk man ncon ustrf ndv aux boundname",
            "shape (maxbound)",
            "reader urword",
        ],
        [
            "block packagedata",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block packagedata",
            "name cellid",
            "type integer",
            "shape (ncelldim)",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rlen",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rwid",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rgrd",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rtp",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rbth",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name rhk",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block packagedata",
            "name man",
            "type string",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block packagedata",
            "name ncon",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name ustrf",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block packagedata",
            "name ndv",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block packagedata",
            "name aux",
            "type double precision",
            "in_record true",
            "tagged false",
            "shape (naux)",
            "reader urword",
            "time_series true",
            "optional true",
        ],
        [
            "block packagedata",
            "name boundname",
            "type string",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "optional true",
        ],
        [
            "block crosssections",
            "name crosssections",
            "type recarray ifno tab6 filein tab6_filename",
            "shape",
            "valid",
            "optional false",
            "reader urword",
        ],
        [
            "block crosssections",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block crosssections",
            "name tab6",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block crosssections",
            "name filein",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block crosssections",
            "name tab6_filename",
            "type string",
            "preserve_case true",
            "in_record true",
            "reader urword",
            "optional false",
            "tagged false",
        ],
        [
            "block connectiondata",
            "name connectiondata",
            "type recarray ifno ic",
            "shape (maxbound)",
            "reader urword",
        ],
        [
            "block connectiondata",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block connectiondata",
            "name ic",
            "type integer",
            "shape (ncon(ifno))",
            "tagged false",
            "in_record true",
            "reader urword",
            "optional true",
            "numeric_index true",
            "support_negative_index true",
        ],
        [
            "block diversions",
            "name diversions",
            "type recarray ifno idv iconr cprior",
            "shape (maxbound)",
            "reader urword",
        ],
        [
            "block diversions",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block diversions",
            "name idv",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block diversions",
            "name iconr",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block diversions",
            "name cprior",
            "type string",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block initialstages",
            "name initialstages",
            "type recarray ifno initialstage",
            "shape (maxbound)",
            "valid",
            "optional false",
            "reader urword",
        ],
        [
            "block initialstages",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "optional false",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block initialstages",
            "name initialstage",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "optional false",
            "reader urword",
        ],
        [
            "block period",
            "name iper",
            "type integer",
            "block_variable True",
            "in_record true",
            "tagged false",
            "shape",
            "valid",
            "reader urword",
            "optional false",
        ],
        [
            "block period",
            "name perioddata",
            "type recarray ifno sfrsetting",
            "shape",
            "reader urword",
        ],
        [
            "block period",
            "name ifno",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block period",
            "name sfrsetting",
            "type keystring status bedk manning stage inflow rainfall evaporation runoff diversionrecord upstream_fraction cross_sectionrecord auxiliaryrecord",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name status",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name bedk",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name manning",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name stage",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name inflow",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name rainfall",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name evaporation",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name runoff",
            "type string",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name diversionrecord",
            "type record diversion idv divflow",
            "shape",
            "tagged",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name diversion",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name idv",
            "type integer",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "numeric_index true",
        ],
        [
            "block period",
            "name divflow",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
        [
            "block period",
            "name upstream_fraction",
            "type double precision",
            "shape",
            "tagged true",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name cross_sectionrecord",
            "type record cross_section tab6 filein tab6_filename",
            "shape",
            "tagged",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name cross_section",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block period",
            "name tab6",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block period",
            "name filein",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
            "tagged true",
            "optional false",
        ],
        [
            "block period",
            "name tab6_filename",
            "type string",
            "preserve_case true",
            "in_record true",
            "reader urword",
            "optional false",
            "tagged false",
        ],
        [
            "block period",
            "name auxiliaryrecord",
            "type record auxiliary auxname auxval",
            "shape",
            "tagged",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name auxiliary",
            "type keyword",
            "shape",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name auxname",
            "type string",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
        ],
        [
            "block period",
            "name auxval",
            "type double precision",
            "shape",
            "tagged false",
            "in_record true",
            "reader urword",
            "time_series true",
        ],
    ]

    def __init__(
        self,
        model,
        loading_package=False,
        storage=None,
        auxiliary=None,
        boundnames=None,
        print_input=None,
        print_stage=None,
        print_flows=None,
        save_flows=None,
        stage_filerecord=None,
        budget_filerecord=None,
        budgetcsv_filerecord=None,
        package_convergence_filerecord=None,
        timeseries=None,
        observations=None,
        mover=None,
        maximum_picard_iterations=None,
        maximum_iterations=None,
        maximum_depth_change=None,
        unit_conversion=None,
        length_conversion=None,
        time_conversion=None,
        dev_storage_weight=None,
        nreaches=None,
        packagedata=None,
        crosssections=None,
        connectiondata=None,
        diversions=None,
        initialstages=None,
        perioddata=None,
        filename=None,
        pname=None,
        **kwargs,
    ):
        """
        ModflowGwfsfr defines a SFR package.

        Parameters
        ----------
        model
            Model that this package is a part of. Package is automatically
            added to model when it is initialized.
        loading_package : bool
            Do not set this parameter. It is intended for debugging and internal
            processing purposes only.
        storage : keyword
            keyword that activates storage contributions to the stream-flow routing package
            continuity equation.
        auxiliary : [string]
            defines an array of one or more auxiliary variable names.  there is no limit on
            the number of auxiliary variables that can be provided on this line; however,
            lists of information provided in subsequent blocks must have a column of data
            for each auxiliary variable name defined here.   the number of auxiliary
            variables detected on this line determines the value for naux.  comments cannot
            be provided anywhere on this line as they will be interpreted as auxiliary
            variable names.  auxiliary variables may not be used by the package, but they
            will be available for use by other parts of the program.  the program will
            terminate with an error if auxiliary variables are specified on more than one
            line in the options block.
        boundnames : keyword
            keyword to indicate that boundary names may be provided with the list of stream
            reach cells.
        print_input : keyword
            keyword to indicate that the list of stream reach information will be written
            to the listing file immediately after it is read.
        print_stage : keyword
            keyword to indicate that the list of stream reach {#2} will be printed to the
            listing file for every stress period in which 'head print' is specified in
            output control.  if there is no output control option and print_{#3} is
            specified, then {#2} are printed for the last time step of each stress period.
        print_flows : keyword
            keyword to indicate that the list of stream reach flow rates will be printed to
            the listing file for every stress period time step in which 'budget print' is
            specified in output control.  if there is no output control option and
            'print_flows' is specified, then flow rates are printed for the last time step
            of each stress period.
        save_flows : keyword
            keyword to indicate that stream reach flow terms will be written to the file
            specified with 'budget fileout' in output control.
        stage_filerecord : record
        budget_filerecord : record
        budgetcsv_filerecord : record
        package_convergence_filerecord : record
        timeseries : record ts6 filein ts6_filename
            Contains data for the ts package. Data can be passed as a dictionary to the ts
            package with variable names as keys and package data as values. Data for the
            timeseries variable is also acceptable. See ts package documentation for more
            information.
        observations : record obs6 filein obs6_filename
            Contains data for the obs package. Data can be passed as a dictionary to the
            obs package with variable names as keys and package data as values. Data for
            the observations variable is also acceptable. See obs package documentation for
            more information.
        mover : keyword
            keyword to indicate that this instance of the sfr package can be used with the
            water mover (mvr) package.  when the mover option is specified, additional
            memory is allocated within the package to store the available, provided, and
            received water.
        maximum_picard_iterations : integer
            integer value that defines the maximum number of streamflow routing picard
            iterations allowed when solving for reach stages and flows as part of the gwf
            formulate step. picard iterations are used to minimize differences in sfr
            package results between subsequent gwf picard (non-linear) iterations as a
            result of non-optimal reach numbering. if reaches are numbered in order, from
            upstream to downstream, maximum_picard_iterations can be set to 1 to reduce
            model run time. by default, maximum_picard_iterations is equal to 100.
        maximum_iterations : integer
            integer value that defines the maximum number of streamflow routing newton-
            raphson iterations allowed for a reach. by default, maximum_iterations is equal
            to 100. maximum_iterations would only need to be increased from the default
            value if one or more reach in a simulation has a large water budget error.
        maximum_depth_change : double precision
            real value that defines the depth closure tolerance. by default,
            maximum_depth_change is equal to :math:`1 times 10^{-5}`. the
            maximum_stage_change would only need to be increased or decreased from the
            default value if the water budget error for one or more reach is too small or
            too large, respectively.
        unit_conversion : double precision
            real value that is used to convert user-specified manning's roughness
            coefficients from seconds per meters:math:`^{1/3}` to model length and time
            units. a constant of 1.486 is used for flow units of cubic feet per second, and
            a constant of 1.0 is used for units of cubic meters per second. the constant
            must be multiplied by 86,400 when using time units of days in the simulation.
        length_conversion : double precision
            real value that is used to convert user-specified manning's roughness
            coefficients from meters to model length units. length_conversion should be set
            to 3.28081, 1.0, and 100.0 when using length units (length_units) of feet,
            meters, or centimeters in the simulation, respectively. length_conversion does
            not need to be specified if length_units are meters.
        time_conversion : double precision
            real value that is used to convert user-specified manning's roughness
            coefficients from seconds to model time units. time_conversion should be set to
            1.0, 60.0, 3,600.0, 86,400.0, and 31,557,600.0 when using time units
            (time_units) of seconds, minutes, hours, days, or years in the simulation,
            respectively. time_conversion does not need to be specified if time_units are
            seconds.
        dev_storage_weight : double precision
            real number value that defines the time weighting factor used to calculate the
            change in channel storage. storage_weight must have a value between 0.5 and 1.
            default storage_weight value is 1.
        nreaches : integer
            integer value specifying the number of stream reaches.  there must be nreaches
            entries in the packagedata block.
        packagedata : [list]
        crosssections : list
        connectiondata : [list]
        diversions : [list]
        initialstages : [list]
        perioddata : list

        filename : str
            File name for this package.
        pname : str
            Package name for this package.
        parent_file : MFPackage
            Parent package file that references this package. Only needed for
            utility packages (mfutl*). For example, mfutllaktab package must have
            a mfgwflak package parent_file.
        """

        super().__init__(model, "sfr", filename, pname, loading_package, **kwargs)

        self.storage = self.build_mfdata("storage", storage)
        self.auxiliary = self.build_mfdata("auxiliary", auxiliary)
        self.boundnames = self.build_mfdata("boundnames", boundnames)
        self.print_input = self.build_mfdata("print_input", print_input)
        self.print_stage = self.build_mfdata("print_stage", print_stage)
        self.print_flows = self.build_mfdata("print_flows", print_flows)
        self.save_flows = self.build_mfdata("save_flows", save_flows)
        self.stage_filerecord = self.build_mfdata("stage_filerecord", stage_filerecord)
        self.budget_filerecord = self.build_mfdata(
            "budget_filerecord", budget_filerecord
        )
        self.budgetcsv_filerecord = self.build_mfdata(
            "budgetcsv_filerecord", budgetcsv_filerecord
        )
        self.package_convergence_filerecord = self.build_mfdata(
            "package_convergence_filerecord", package_convergence_filerecord
        )
        self._ts_filerecord = self.build_mfdata("ts_filerecord", None)
        self._ts_package = self.build_child_package(
            "ts", timeseries, "timeseries", self._ts_filerecord
        )
        self._obs_filerecord = self.build_mfdata("obs_filerecord", None)
        self._obs_package = self.build_child_package(
            "obs", observations, "continuous", self._obs_filerecord
        )
        self.mover = self.build_mfdata("mover", mover)
        self.maximum_picard_iterations = self.build_mfdata(
            "maximum_picard_iterations", maximum_picard_iterations
        )
        self.maximum_iterations = self.build_mfdata(
            "maximum_iterations", maximum_iterations
        )
        self.maximum_depth_change = self.build_mfdata(
            "maximum_depth_change", maximum_depth_change
        )
        self.unit_conversion = self.build_mfdata("unit_conversion", unit_conversion)
        self.length_conversion = self.build_mfdata(
            "length_conversion", length_conversion
        )
        self.time_conversion = self.build_mfdata("time_conversion", time_conversion)
        self.dev_storage_weight = self.build_mfdata(
            "dev_storage_weight", dev_storage_weight
        )
        self.nreaches = self.build_mfdata("nreaches", nreaches)
        self.packagedata = self.build_mfdata("packagedata", packagedata)
        self.crosssections = self.build_mfdata("crosssections", crosssections)
        self.connectiondata = self.build_mfdata("connectiondata", connectiondata)
        self.diversions = self.build_mfdata("diversions", diversions)
        self.initialstages = self.build_mfdata("initialstages", initialstages)
        self.perioddata = self.build_mfdata("perioddata", perioddata)

        self._init_complete = True
